@@ -25,14 +25,24 @@
 import httplib2
 import json
 import logging
+from urllib import quote
+
+
+class ChronosAPIError(Exception):
+    pass
+
+
+class MissingFieldError(Exception):
+    pass
 
 
 class ChronosClient(object):
     _user = None
     _password = None
 
-    def __init__(self, hostname, proto="http", username=None, password=None, level='WARN'):
-        self.baseurl = "%s://%s" % (proto, hostname)
+    def __init__(self, servers, proto="http", username=None, password=None, level='WARN'):
+        server_list = servers if isinstance(servers, list) else [servers]
+        self.servers = ["%s://%s" % (proto, server) for server in server_list]
         if username and password:
             self._user = username
             self._password = password
@@ -74,6 +84,28 @@ class ChronosClient(object):
         """Update an existing job by name"""
         return self.add(job_def, update=True)
 
+    def job_stat(self, name):
+        """ List stats for a job """
+        return self._call('/scheduler/job/stat/%s' % name, "GET")
+
+    def scheduler_stat_99th(self):
+        return self._call('/scheduler/stats/99thPercentile', 'GET')
+
+    def scheduler_stat_98th(self):
+        return self._call('/scheduler/stats/98thPercentile', 'GET')
+
+    def scheduler_stat_95th(self):
+        return self._call('/scheduler/stats/95thPercentile', 'GET')
+
+    def scheduler_stat_75th(self):
+        return self._call('/scheduler/stats/75thPercentile', 'GET')
+
+    def scheduler_stat_median(self):
+        return self._call('/scheduler/stats/median', 'GET')
+
+    def scheduler_stat_mean(self):
+        return self._call('/scheduler/stats/mean', 'GET')
+
     def _call(self, url, method="GET", body=None, headers={}):
         hdrs = {}
         if body:
@@ -85,9 +117,21 @@ class ChronosClient(object):
         conn = httplib2.Http(disable_ssl_certificate_validation=True)
         if self._user and self._password:
             conn.add_credentials(self._user, self._password)
-        endpoint = "%s%s" % (self.baseurl, url)
-        self.logger.debug(endpoint)
-        return self._check(*conn.request(endpoint, method, body=body, headers=hdrs))
+
+        response = None
+        servers = list(self.servers)
+        while servers:
+            server = servers.pop(0)
+            endpoint = "%s%s" % (server, quote(url))
+            self.logger.debug(endpoint)
+            try:
+                response = self._check(*conn.request(endpoint, method, body=body, headers=hdrs))
+                self.logger.info('Got response from %s', endpoint)
+                return response
+            except Exception as e:
+                self.logger.error('Error while calling %s: %s', endpoint, e.message)
+
+        raise ChronosAPIError('No remaining Chronos servers to try')
 
     def _check(self, resp, content):
         status = resp.status
@@ -101,18 +145,18 @@ class ChronosClient(object):
                 payload = content
 
         if payload is None and status != 204:
-            raise Exception("HTTP Error %d occurred." % status)
+            raise ChronosAPIError("Request to Chronos API failed: status: %d, response: %s" % (status, content))
 
         return payload
 
     def _check_fields(self, job):
         for k in ChronosJob.fields:
             if k not in job:
-                raise Exception("missing required field %s" % k)
+                raise MissingFieldError("missing required field %s" % k)
         for k in ChronosJob.one_of:
             if k in job:
                 return True
-        raise Exception("Job must include one of %s" % ChronosJob.one_of)
+        raise MissingFieldError("Job must include one of %s" % ChronosJob.one_of)
 
 
 class ChronosJob(object):
@@ -127,5 +171,5 @@ class ChronosJob(object):
     one_of = ["schedule", "parents"]
 
 
-def connect(hostname, proto="http", username=None, password=None):
-    return ChronosClient(hostname, proto="http", username=username, password=password)
+def connect(servers, proto="http", username=None, password=None):
+    return ChronosClient(servers, proto=proto, username=username, password=password)
